
/**
 * This is a specialized node for the tree that I needed to create.  Each node has five links
 * since there are five actions that can be done from each given state.  Not all new states
 * are necessarily legal, but there would be a max of 5 new nodes added for each given
 * state.  I also added a link to the node's parent node for easier access to the node's
 * parent.  Also, each node holds an array of size three which corresponds to a state.
 * 
 * @author Mark Brown
 * @course AI
 * @version January 27, 2004 - Project 1
 */

public class TreeNode extends ListItem
{
    /*** Instance Variables ***/
    public TreeNode[] links;              // links to new states
    public TreeNode plink;              // link to parent node
    public int MMValue;                     // minimax value
    public int numOfChildren;
    public CheckerBoard entry;
    public int[] moves;
    
    /*** Constructors ***/
    public TreeNode(CheckerBoard board)
    {
        links = new TreeNode[24];
        for (int i = 0; i < 24; i++)
        {
            links[i] = null;
        }
        plink = null;
        numOfChildren = 0;
        entry = board;
        MMValue = entry.MMFinder();
        moves = new int[4];
    }
    
    /*** Instance Methods ***/
    // Add a child to a given node
    public void addChild(TreeNode newNode)
    {
        links[numOfChildren] = newNode;
        numOfChildren++;
        newNode.plink = this;
    }
    
    public TreeNode copy(TreeNode newNode)
    {
        CheckerBoard tempBoard = new CheckerBoard(entry.copy(newNode.entry.playingBoard));
        TreeNode tempNode = new TreeNode(tempBoard);
        tempNode.plink = newNode.plink;
        tempNode.links = newNode.links;
        tempNode.MMValue = newNode.MMValue;
        tempNode.numOfChildren = newNode.numOfChildren;
        tempNode.entry.beginningX = newNode.entry.beginningX;
        tempNode.entry.beginningY = newNode.entry.beginningY;
        tempNode.entry.endingX = newNode.entry.endingX;
        tempNode.entry.endingY = newNode.entry.endingY;
        for (int i = 0; i < newNode.entry.attackArray.length; i++)
        {
            tempNode.entry.attackArray[i] = newNode.entry.attackArray[i];
        }
        
        return tempNode;
    }
    
    // isLeaf:  returns true if this node is a leaf node.
    /*public boolean isLeaf()
    {
        if (link1==null && link2==null && link3==null && link4==null &&
            link5==null)
        {
            return true;
        }
        else
            return false;
    }*/// end of isLeaf
}
